@@ -69,7 +69,7 @@ our @EXPORT = qw(
 	SASS2SCSS_CONVERT_COMMENT
 );
 
-our $VERSION = "3.4.13";
+our $VERSION = "3.6.0";
 
 require XSLoader;
 XSLoader::load('CSS::Sass', $VERSION);
@@ -632,7 +632,7 @@ automatically recognize the format of your string data.
 
 L<CSS::Sass::Value>
 
-L<The Sass Home Page|http://sass-lang.com/>
+L<The Sass Home Page|https://sass-lang.com/>
 
 L<The libsass Home Page|https://github.com/sass/libsass>
 
