@@ -2,12 +2,14 @@
 #define SASS_BIND_H
 
 #include <string>
+#include "backtrace.hpp"
 #include "environment.hpp"
 #include "ast_fwd_decl.hpp"
 
 namespace Sass {
 
-  void bind(std::string type, std::string name, Parameters_Obj, Arguments_Obj, Context*, Env*, Eval*);
+  void bind(std::string type, std::string name, Parameters_Obj, Arguments_Obj, Env*, Eval*, Backtraces& traces);
+
 }
 
 #endif
