@@ -7,6 +7,7 @@
 
 namespace Sass {
 
+  // this defeats the whole purpose of environment being templatable!!
   typedef environment_map<std::string, AST_Node_Obj>::iterator EnvIter;
 
   class EnvResult {
@@ -92,6 +93,10 @@ namespace Sass {
     // include all scopes available
     bool has(const std::string& key) const;
 
+    // look on the full stack for key
+    // include all scopes available
+    T& get(const std::string& key);
+
     // look on the full stack for key
     // include all scopes available
     EnvResult find(const std::string& key);
@@ -107,6 +112,7 @@ namespace Sass {
 
   // define typedef for our use case
   typedef Environment<AST_Node_Obj> Env;
+  typedef std::vector<Env*> EnvStack;
 
 }
 
