@@ -17,7 +17,7 @@ namespace Sass {
   #ifdef DEBUG_SHARED_PTR
 
     #define SASS_MEMORY_NEW(Class, ...) \
-      ((new Class(__VA_ARGS__))->trace(__FILE__, __LINE__)) \
+      ((Class*)(new Class(__VA_ARGS__))->trace(__FILE__, __LINE__)) \
 
     #define SASS_MEMORY_COPY(obj) \
       ((obj)->copy(__FILE__, __LINE__)) \
@@ -78,6 +78,9 @@ namespace Sass {
     static void setTaint(bool val) {
       taint = val;
     }
+
+    virtual const std::string to_string() const = 0;
+
     virtual ~SharedObj();
     long getRefCount() {
       return refcounter;
@@ -169,6 +172,14 @@ namespace Sass {
       }
       return *this;
     }
+
+    // allow implicit conversion to string
+    // relies on base class implementation
+    operator const std::string() const {
+      if (node) return node->to_string();
+      else return std::string("[NULLPTR]");
+    }
+
     ~SharedImpl() {};
   public:
     operator T*() const {
@@ -203,4 +214,4 @@ namespace Sass {
 
 }
 
-#endif
\ No newline at end of file
+#endif
