@@ -25,6 +25,7 @@ namespace Sass {
     const char* kwd_gte(const char* src);
     const char* kwd_lt(const char* src);
     const char* kwd_lte(const char* src);
+    const char* kwd_using(const char* src);
 
     // Match standard control chars
     const char* kwd_at(const char* src);
@@ -64,16 +65,15 @@ namespace Sass {
       size_t level = 0;
       bool in_squote = false;
       bool in_dquote = false;
-      // bool in_braces = false;
-
-      while (*src) {
-
-        // check for abort condition
-        if (end && src >= end) break;
+      bool in_backslash_escape = false;
 
+      while ((end == nullptr || src < end) && *src != '\0') {
         // has escaped sequence?
-        if (*src == '\\') {
-          ++ src; // skip this (and next)
+        if (in_backslash_escape) {
+          in_backslash_escape = false;
+        }
+        else if (*src == '\\') {
+          in_backslash_escape = true;
         }
         else if (*src == '"') {
           in_dquote = ! in_dquote;
@@ -92,14 +92,14 @@ namespace Sass {
         }
 
         // look for the closer (maybe final, maybe not)
-        else if (const char* final = stop(src)) {
+        else if (const char* last = stop(src)) {
           // only close one level?
           if (level > 0) -- level;
           // return position at end of stop
           // delimiter may be multiple chars
-          else return final;
+          else return last;
           // advance position
-          src = final - 1;
+          src = last - 1;
         }
 
         // next
@@ -119,7 +119,7 @@ namespace Sass {
     // first start/opener must be consumed already!
     template<prelexer start, prelexer stop>
     const char* skip_over_scopes(const char* src) {
-      return skip_over_scopes<start, stop>(src, 0);
+      return skip_over_scopes<start, stop>(src, nullptr);
     }
 
     // Match a sequence of characters delimited by the supplied chars.
@@ -264,7 +264,6 @@ namespace Sass {
     const char* kwd_while_directive(const char* src);
 
     const char* re_nothing(const char* src);
-    const char* re_type_selector2(const char* src);
 
     const char* re_special_fun(const char* src);
 
@@ -355,6 +354,10 @@ namespace Sass {
     const char* ie_keyword_arg_value(const char* src);
     const char* ie_keyword_arg_property(const char* src);
 
+    // characters that terminate parsing of a list
+    const char* list_terminator(const char* src);
+    const char* space_list_terminator(const char* src);
+
     // match url()
     const char* H(const char* src);
     const char* W(const char* src);
@@ -362,6 +365,7 @@ namespace Sass {
     const char* UUNICODE(const char* src);
     const char* NONASCII(const char* src);
     const char* ESCAPE(const char* src);
+    const char* real_uri(const char* src);
     const char* real_uri_suffix(const char* src);
     // const char* real_uri_prefix(const char* src);
     const char* real_uri_value(const char* src);
@@ -376,6 +380,9 @@ namespace Sass {
     const char* static_property(const char* src);
     const char* static_value(const char* src);
 
+    const char* css_variable_value(const char* src);
+    const char* css_variable_top_level_value(const char* src);
+
     // Utility functions for finding and counting characters in a string.
     template<char c>
     const char* find_first(const char* src) {
