@@ -1,4 +1,7 @@
+// sass.hpp must go before all system headers to get the
+// __EXTENSIONS__ fix on Solaris.
 #include "sass.hpp"
+
 #include "sass.h"
 #include "values.hpp"
 
@@ -7,22 +10,27 @@
 namespace Sass {
 
   // convert value from C++ side to C-API
-  union Sass_Value* ast_node_to_sass_value (const Expression_Ptr val)
+  union Sass_Value* ast_node_to_sass_value (const Expression* val)
   {
     if (val->concrete_type() == Expression::NUMBER)
     {
-      Number_Ptr_Const res = Cast<Number>(val);
+      const Number* res = Cast<Number>(val);
       return sass_make_number(res->value(), res->unit().c_str());
     }
     else if (val->concrete_type() == Expression::COLOR)
     {
-      Color_Ptr_Const col = Cast<Color>(val);
-      return sass_make_color(col->r(), col->g(), col->b(), col->a());
+      if (const Color_RGBA* rgba = Cast<Color_RGBA>(val)) {
+        return sass_make_color(rgba->r(), rgba->g(), rgba->b(), rgba->a());
+      } else {
+        // ToDo: allow to also use HSLA colors!!
+        Color_RGBA_Obj col = Cast<Color>(val)->copyAsRGBA();
+        return sass_make_color(col->r(), col->g(), col->b(), col->a());
+      }
     }
     else if (val->concrete_type() == Expression::LIST)
     {
-      List_Ptr_Const l = Cast<List>(val);
-      union Sass_Value* list = sass_make_list(l->size(), l->separator());
+      const List* l = Cast<List>(val);
+      union Sass_Value* list = sass_make_list(l->size(), l->separator(), l->is_bracketed());
       for (size_t i = 0, L = l->length(); i < L; ++i) {
         Expression_Obj obj = l->at(i);
         auto val = ast_node_to_sass_value(obj);
@@ -32,7 +40,7 @@ namespace Sass {
     }
     else if (val->concrete_type() == Expression::MAP)
     {
-      Map_Ptr_Const m = Cast<Map>(val);
+      const Map* m = Cast<Map>(val);
       union Sass_Value* map = sass_make_map(m->length());
       size_t i = 0; for (auto __key = (m->keys()).begin(); __key != (m->keys()).end(); ++__key) { Expression_Obj key = *(__key);
         sass_map_set_key(map, i, ast_node_to_sass_value(key));
@@ -47,16 +55,16 @@ namespace Sass {
     }
     else if (val->concrete_type() == Expression::BOOLEAN)
     {
-      Boolean_Ptr_Const res = Cast<Boolean>(val);
+      const Boolean* res = Cast<Boolean>(val);
       return sass_make_boolean(res->value());
     }
     else if (val->concrete_type() == Expression::STRING)
     {
-      if (String_Quoted_Ptr_Const qstr = Cast<String_Quoted>(val))
+      if (const String_Quoted* qstr = Cast<String_Quoted>(val))
       {
         return sass_make_qstring(qstr->value().c_str());
       }
-      else if (String_Constant_Ptr_Const cstr = Cast<String_Constant>(val))
+      else if (const String_Constant* cstr = Cast<String_Constant>(val))
       {
         return sass_make_string(cstr->value().c_str());
       }
@@ -65,7 +73,7 @@ namespace Sass {
   }
 
   // convert value from C-API to C++ side
-  Value_Ptr sass_value_to_ast_node (const union Sass_Value* val)
+  Value* sass_value_to_ast_node (const union Sass_Value* val)
   {
     switch (sass_value_get_tag(val)) {
       case SASS_NUMBER:
@@ -78,7 +86,8 @@ namespace Sass {
                                ParserState("[C-VALUE]"),
                                sass_boolean_get_value(val));
       case SASS_COLOR:
-        return SASS_MEMORY_NEW(Color,
+        // ToDo: allow to also use HSLA colors!!
+        return SASS_MEMORY_NEW(Color_RGBA,
                                ParserState("[C-VALUE]"),
                                sass_color_get_r(val),
                                sass_color_get_g(val),
@@ -94,17 +103,18 @@ namespace Sass {
                                  ParserState("[C-VALUE]"),
                                  sass_string_get_value(val));
       case SASS_LIST: {
-        List_Ptr l = SASS_MEMORY_NEW(List,
+        List* l = SASS_MEMORY_NEW(List,
                                   ParserState("[C-VALUE]"),
                                   sass_list_get_length(val),
                                   sass_list_get_separator(val));
         for (size_t i = 0, L = sass_list_get_length(val); i < L; ++i) {
           l->append(sass_value_to_ast_node(sass_list_get_value(val, i)));
         }
+        l->is_bracketed(sass_list_get_is_bracketed(val));
         return l;
       }
       case SASS_MAP: {
-        Map_Ptr m = SASS_MEMORY_NEW(Map, ParserState("[C-VALUE]"));
+        Map* m = SASS_MEMORY_NEW(Map, ParserState("[C-VALUE]"));
         for (size_t i = 0, L = sass_map_get_length(val); i < L; ++i) {
           *m << std::make_pair(
             sass_value_to_ast_node(sass_map_get_key(val, i)),
