@@ -5,7 +5,7 @@ module SassSpec::CLI
 
   def self.parse
     options = {
-      engine_adapter: SassEngineAdapter.new,
+      engine_adapter: nil,
       spec_directory: nil,
       generate: false,
       tap: false,
@@ -13,7 +13,6 @@ module SassSpec::CLI
       verbose: false,
       filter: "",
       limit: -1,
-      only_output_styles: []
     }
 
     OptionParser.new do |opts|
@@ -43,24 +42,25 @@ Make sure the command you provide prints to stdout.
         options[:verbose] = true
       end
 
-      opts.on("-V", "--version LANGUAGE_VERSION", "Select the Sass Language Version to test.") do |v|
-        unless SassSpec::LANGUAGE_VERSIONS.include?(v)
-          raise ArgumentError.new("Version #{v} is not valid. " +
-                                  "Did you mean one of: #{SassSpec::LANGUAGE_VERSIONS.join(', ')}")
-        end
-        options[:language_version] = v
-      end
-
       opts.on("-t", "--tap", "Output TAP compatible report") do
         options[:tap] = true
       end
 
-      opts.on("-c", "--command COMMAND", "Sets a specific binary to run (defaults to '#{options[:engine_adapter]}')") do |v|
+      opts.on("-c", "--command COMMAND", "Sets a specific binary to run") do |v|
         options[:engine_adapter] = ExecutableEngineAdapter.new(v)
       end
 
       opts.on("--dart PATH", "Run Dart Sass, whose repo should be at the given path.") do |path|
         options[:engine_adapter] = DartEngineAdapter.new(path)
+        options[:engine_adapter].args = options[:dart_args]
+      end
+
+      opts.on("--dart-args ARGS", "Pass ARGS to Dart Sass.") do |args|
+        if (adapter = options[:engine_adapter]) && adapter.is_a?(DartEngineAdapter)
+          adapter.args = args
+        else
+          options[:dart_args] = args
+        end
       end
 
       opts.on("-g", "--generate", "Run test(s) and generate expected output file(s).") do
@@ -87,24 +87,6 @@ Make sure the command you provide prints to stdout.
         options[:limit] = limit.to_i
       end
 
-      opts.on("--output-style STYLE", [:expanded, :compact, :nested, :compressed, :unspecified],
-              "Only run tests that have the specified output style.",
-              "Legal values: expanded, compact, nested, compressed, unspecified.") do |style|
-        style = nil if style == :unspecified
-        options[:only_output_styles] << style
-      end
-
-      opts.on("-r SPEC_DIR", "--root SPEC_DIR", "Root directory for the specs. ",
-                "Defaults to the first directory specified if not provided or ",
-                "the default spec directory if no directory is specified or if the first directory",
-                "specified is a subdirectory of the default spec directory.") do |spec_dir|
-        options[:spec_directory] = File.expand_path(spec_dir)
-      end
-
-      opts.on("--migrate-version", "Copy tests that fail and make them pass for the current version.") do
-        options[:migrate_version] = true
-      end
-
       opts.on("--migrate-impl", "Copy tests that fail and make them pass for the current implementatino.") do
         options[:migrate_impl] = true
       end
@@ -122,15 +104,7 @@ Make sure the command you provide prints to stdout.
       end
     end.parse!
 
-    if ARGV.any? && !options[:spec_directory]
-      if File.expand_path(ARGV[0]).start_with?(SassSpec::SPEC_DIR)
-        options[:spec_directory] = SassSpec::SPEC_DIR
-      else
-        options[:spec_directory] = File.expand_path(ARGV[0])
-      end
-    end
-
-    options[:spec_directory] ||= SassSpec::SPEC_DIR
+    raise "Must specify either --dart or --command." if options[:engine_adapter].nil?
 
     options[:spec_dirs_to_run] = ARGV.dup.map{|d| File.expand_path(d)} if ARGV.any?
 
