@@ -3,6 +3,7 @@ require 'thread'
 require 'fileutils'
 require 'yaml'
 require_relative "interactor"
+require_relative "util"
 
 # Holder to put and run test cases
 class SassSpec::Test < Minitest::Test
@@ -32,16 +33,19 @@ class SassSpecRunner
 
   def run
     if @test_case.todo? && !@options[:run_todo]
-      skip "Skipped #{@test_case.folder}"
+      skip "Skipped #{@test_case.name}"
     end
 
-    assert_filename_length!(@test_case.input_path)
-    assert_filename_length!(@test_case.expected_path)
+    unless @test_case.dir.hrx?
+      @test_case.dir.glob("*").each {|p| assert_filename_length!(File.join(@test_case.dir.path, p))}
+    end
 
-    assert File.exists?(@test_case.input_path),
-      "Input #{@test_case.input_path} file does not exist"
+    @output, @error, @status = @test_case.run(@options[:engine_adapter])
+    @output = @output.gsub(/\r\n/, "\n")
+    @normalized_output = SassSpec::Util.normalize_output(@output)
+    @error = SassSpec::Util.normalize_error(@error)
 
-    if @test_case.overwrite?
+    if @options[:generate]
       overwrite_test!
       return true
     end
@@ -51,20 +55,16 @@ class SassSpecRunner
     # checks in nested functions without worrying about piping return values.
     catch :done do
       check_annotations!
+      handle_conflicting_files!
       handle_missing_output!
       handle_unexpected_error!
       handle_unexpected_pass!
       handle_output_difference!
 
-      if @test_case.ignore_warning?
-        return true
-      elsif @test_case.warning_todo? && !@options[:run_todo]
-        skip "Skipped warning check for #{@test_case.folder}"
-      else
-        handle_expected_error_message!
-        handle_unexpected_error_message!
-      end
-
+      # Run these checks last because `handle_stderr_difference!` can skip the
+      # test if `@test_case.warning_todo?` is set, and we only want to check for
+      # an unnecessary TODO if the test isn't skipped because of a TODO.
+      handle_stderr_difference!
       handle_unnecessary_todo!
     end
 
@@ -78,10 +78,9 @@ class SassSpecRunner
   def check_annotations!
     return unless @options[:check_annotations]
 
-    _output, _, error, _ = @test_case.output
     ignored_warning_impls = @test_case.metadata.warnings_ignored_for
 
-    if ignored_warning_impls.any? && error.length == 0
+    if ignored_warning_impls.any? && @error.empty?
       message = "No warning issued, but warnings are ignored for #{ignored_warning_impls.join(', ')}"
       choice = interact(:ignore_warning_nonexistant, :fail) do |i|
         i.prompt message
@@ -97,7 +96,7 @@ class SassSpecRunner
     end
 
     todo_warning_impls = @test_case.metadata.all_warning_todos
-    if todo_warning_impls.any? && error.length == 0
+    if todo_warning_impls.any? && @error.length == 0
       message = "No warning issued, but warnings are pending for #{todo_warning_impls.join(', ')}"
       choice = interact(:todo_warning_nonexistant, :fail) do |i|
         i.prompt message
@@ -113,28 +112,58 @@ class SassSpecRunner
     end
   end
 
-  def handle_missing_output!
-    return if File.exists?(@test_case.expected_path)
+  def handle_conflicting_files!
+    if @test_case.file?("error", impl: true)
+      impl = true
+    elsif @test_case.file?("error")
+      impl = false
+    else
+      return
+    end
 
-    output, _, error, _ = @test_case.output
+    output_file_exists = @test_case.file?("output.css", impl: impl)
+    warning_file_exists = @test_case.file?("warning", impl: impl)
+    return unless output_file_exists || warning_file_exists
 
-    skip_test_case!("TODO test is failing") if @test_case.probe_todo?
-
-    choice = interact(:missing_output, :fail) do |i|
-      i.prompt "Expected output file does not exist."
+    choice = interact(:conflicting_files, :fail) do |i|
+      i.prompt "Test has both error and success outputs."
 
       show_input_choice(i)
+      show_output_choice(i)
 
-      i.choice('e', "Show me the #{error.length > 0 ? 'error' : 'output'} generated.") do
-        to_show = error
-        to_show = output if to_show.length == 0
-        to_show = "No output or error to display." if to_show.length == 0
+      if output_file_exists
+        i.choice('eo', 'Show expected output.') do
+          display_text_block(@test_case.read("output.css", impl: impl))
+          i.restart!
+        end
+      end
 
-        display_text_block(to_show)
+      if warning_file_exists
+        i.choice('ew', 'Show expected warning.') do
+          display_text_block(@test_case.read("warning", impl: impl))
+          i.restart!
+        end
+      end
+
+      i.choice('ee', 'Show expected error.') do
+        display_text_block(@test_case.read("error", impl: :auto))
         i.restart!
       end
 
       delete_choice(i)
+
+      i.choice('S', 'Keep the success output.') do
+        @test_case.delete("error", impl: :auto)
+        throw :done
+      end
+
+      i.choice('E', 'Keep the error output.') do
+        @test_case.delete("output.css") if output_file_exists
+        @test_case.delete("warning") if warning_file_exists
+        throw :done
+      end
+
+      migrate_warning_choice(i) unless warning_file_exists
       update_output_choice(i)
       fail_or_exit_choice(i)
     end
@@ -142,21 +171,33 @@ class SassSpecRunner
     assert choice != :fail, "Expected #{@test_case.expected_path} file does not exist"
   end
 
-  def handle_unexpected_error!
-    _output, _clean_output, error, status = @test_case.output
-    return if status == 0 || @test_case.should_fail?
+  def handle_missing_output!
+    return if @test_case.should_fail? || @test_case.expected
 
-    unless @test_case.interactive?
-      if @test_case.migrate_version?
-        migrate_version!
-        throw :done
-      elsif @test_case.migrate_impl?
-        migrate_impl!
-        throw :done
-      end
+    skip_test_case!("TODO test is failing") if probe_todo?
+
+    choice = interact(:missing_output, :fail) do |i|
+      i.prompt "Expected output file does not exist."
+
+      show_input_choice(i)
+      show_output_choice(i)
+      delete_choice(i)
+      update_output_choice(i)
+      fail_or_exit_choice(i)
     end
 
-    skip_test_case!("TODO test is failing") if @test_case.probe_todo?
+    assert choice != :fail, "Expected output.css file does not exist"
+  end
+
+  def handle_unexpected_error!
+    return if @status == 0 || @test_case.should_fail?
+
+    if !@options[:interactive] && @options[:migrate_impl]
+      migrate_impl!
+      throw :done
+    end
+
+    skip_test_case!("TODO test is failing") if @test_case.todo?
 
     choice = interact(:unexpected_error, :fail) do |i|
       i.prompt "An unexpected compiler error was encountered."
@@ -164,12 +205,11 @@ class SassSpecRunner
       show_input_choice(i)
 
       i.choice('e', "Show me the error.") do
-        display_text_block(error)
+        display_text_block(@error)
         i.restart!
       end
 
       update_output_choice(i)
-      migrate_version_choice(i)
       migrate_impl_choice(i)
       todo_choice(i)
       ignore_choice(i)
@@ -177,29 +217,19 @@ class SassSpecRunner
     end
     throw :done unless choice == :fail
 
-    assert_equal 0, status,
-      "Command `#{@options[:engine_adapter]}` did not complete:\n\n#{error}"
+    assert_equal 0, @status,
+      "Command `#{@options[:engine_adapter]}` did not complete:\n\n#{@error}"
   end
 
   def handle_unexpected_pass!
-    output, _clean_output, _error, status = @test_case.output
-    return if status != 0
+    return unless @status == 0 && @test_case.should_fail?
 
-    if @test_case.interactive?
-      return if !@test_case.should_fail?
-    else
-      return if !@test_case.should_fail?
-
-      if @test_case.migrate_version?
-        migrate_version!
-        throw :done
-      elsif @test_case.migrate_impl?
-        migrate_impl!
-        throw :done
-      end
+    if !@options[:interactive] && @options[:migrate_impl]
+      migrate_impl!
+      throw :done
     end
 
-    skip_test_case!("TODO test is failing") if @test_case.probe_todo?
+    skip_test_case!("TODO test is failing") if probe_todo?
 
     choice = interact(:unexpected_pass, :fail) do |i|
       i.prompt "A failure was expected but it compiled instead."
@@ -207,46 +237,38 @@ class SassSpecRunner
       show_input_choice(i)
 
       i.choice('e', "Show me the expected error.") do
-        display_text_block(File.read(@test_case.error_path))
+        display_text_block(@test_case.expected_error)
         i.restart!
       end
 
       i.choice('o', "Show me the output.") do
-        display_text_block(output)
+        display_text_block(@output)
         i.restart!
       end
 
+      migrate_warning_choice(i)
       update_output_choice(i)
-      migrate_version_choice(i)
       migrate_impl_choice(i)
       todo_choice(i)
       fail_or_exit_choice(i)
     end
     throw :done unless choice == :fail
 
-    # XXX Ruby returns 65 etc. SassC returns 1
-    refute_equal status, 0, "Test case should fail, but it did not"
+    refute_equal @status, 0, "Test case should fail, but it did not"
   end
 
   def handle_output_difference!
-    _output, clean_output, _error, _status = @test_case.output
+    return if @test_case.should_fail? || @normalized_output == @test_case.expected
 
-    return if @test_case.expected == clean_output
-
-    unless @test_case.interactive?
-      if @test_case.migrate_version?
-        migrate_version!
-        throw :done
-      elsif @test_case.migrate_impl?
-        migrate_impl!
-        throw :done
-      end
+    if !@options[:interactive] && @options[:migrate_impl]
+      migrate_impl!
+      throw :done
     end
 
-    skip_test_case!("TODO test is failing") if @test_case.probe_todo?
+    skip_test_case!("TODO test is failing") if probe_todo?
 
     interact(:output_difference, :fail) do |i|
-      i.prompt "output does not match expectation"
+      i.prompt "Output does not match expectation."
 
       show_input_choice(i)
 
@@ -254,44 +276,39 @@ class SassSpecRunner
         require 'diffy'
         display_text_block(
           Diffy::Diff.new("Expected\n" + @test_case.expected,
-                          "Actual\n" + clean_output).to_s(:color))
+                          "Actual\n" + @normalized_output).to_s(:color))
         i.restart!
       end
 
       update_output_choice(i)
-      migrate_version_choice(i)
       migrate_impl_choice(i)
       todo_choice(i)
       ignore_choice(i)
       fail_or_exit_choice(i)
     end
 
-    assert_equal @test_case.expected, clean_output, "expected did not match output"
+    assert_equal @test_case.expected, @normalized_output, "expected did not match output"
   end
 
   def handle_unnecessary_todo!
-    output, clean_output, _error, _status = @test_case.output
-
-    return if @test_case.probe_todo? && !@test_case.interactive?
+    return if probe_todo? && !@options[:interactive]
     return unless @test_case.todo? || @test_case.warning_todo?
 
-    expected_error_msg = extract_error_message(@test_case.expected_error)
-
     interact(:unnecessary_todo, :fail) do |i|
       i.prompt "Test is passing but marked as TODO."
 
       show_input_choice(i)
 
-      unless expected_error_msg.empty?
+      if @test_case.should_fail?
         i.choice('e', "Show me the expected error.") do
-          display_text_block(expected_error_msg)
+          display_text_block(@test_case.expected_error)
           i.restart!
         end
       end
 
-      unless output.empty?
+      unless @output.empty?
         i.choice('o', "Show me the output.") do
-          display_text_block(output)
+          display_text_block(@output)
           i.restart!
         end
       end
@@ -310,35 +327,56 @@ class SassSpecRunner
       end
     end
 
-    assert_equal @test_case.expected, clean_output, "expected did not match output"
+    assert_equal @test_case.expected, @normalized_output, "expected did not match output"
   end
 
-  def handle_expected_error_message!
-    return unless @test_case.verify_stderr?
+  def handle_stderr_difference!
+    unless @test_case.should_fail?
+      if @test_case.ignore_warning?
+        return
+      elsif @test_case.warning_todo? && !@options[:run_todo]
+        skip_test_case! "Skipped warning check for #{@test_case.name}"
+      end
+    end
 
-    _output, _clean_output, error, _status = @test_case.output
-
-    error_msg = extract_error_message(error)
-    expected_error_msg = extract_error_message(@test_case.expected_error)
+    error_msg = extract_error_message(@error)
+    expected_error_msg = extract_error_message(
+      @test_case.expected_error || @test_case.expected_warning)
 
     return if expected_error_msg == error_msg
-    skip_test_case!("TODO test is failing.") if @test_case.probe_todo?
+    skip_test_case!("TODO test is failing") if probe_todo?
 
-    interact(:expected_error_different, :fail) do |i|
-      i.prompt(
-        if error_msg.empty?
+    message =
+      if error_msg.empty?
+        if @test_case.should_fail?
           "An error message was expected but wasn't produced."
         else
-          "Error output doesn't match what was expected."
-        end)
+          "A warning was expected but wasn't produced."
+        end
+      elsif expected_error_msg.empty?
+        "An unexpected warning was produced."
+      else
+        if @test_case.should_fail?
+          "Error message doesn't match the expected error."
+        else
+          "Warning doesn't match the expected warning."
+        end
+      end
+
+    type = @test_case.should_fail? ? :expected_error_different : :expected_warning_different
+    interact(type, :fail) do |i|
+      i.prompt(message)
 
       show_input_choice(i)
 
       if error_msg.empty?
-        return if expected_error_msg.empty?
-
-        i.choice('e', "Show expected error.") do
-          display_text_block(expected_error_msg)
+        i.choice('e', "Show expected #{@test_case.should_fail? ? 'error' : 'warning'}.") do
+          display_text_block(@test.expected_error)
+          i.restart!
+        end
+      elsif expected_error_msg.empty?
+        i.choice('e', "Show #{@test_case.should_fail? ? 'error' : 'warning'}.") do
+          display_text_block(error_msg)
           i.restart!
         end
       else
@@ -352,46 +390,13 @@ class SassSpecRunner
       end
 
       update_output_choice(i)
-      migrate_version_choice(i)
       migrate_impl_choice(i)
       todo_warning_choice(i)
       ignore_warning_choice(i)
       fail_or_exit_choice(i)
     end
 
-    assert_equal expected_error_msg, error_msg,
-      (error_msg.empty? ? "No error message produced" : "Expected did not match error")
-  end
-
-  def handle_unexpected_error_message!
-    return if @test_case.verify_stderr?
-
-    _output, _clean_output, error, _status = @test_case.output
-
-    error_msg = extract_error_message(error)
-
-    return if error_msg.empty?
-
-    skip_test_case!("TODO test is failing") if @test_case.probe_todo?
-
-    interact(:unexpected_error_message, :fail) do |i|
-      i.prompt "Unexpected output to stderr."
-
-      show_input_choice(i)
-
-      i.choice('e', "Show error.") do
-        display_text_block(error)
-        i.restart!
-      end
-
-      update_output_choice(i)
-      migrate_version_choice(i)
-      migrate_impl_choice(i)
-      todo_warning_choice(i)
-      ignore_warning_choice(i)
-      fail_or_exit_choice(i)
-    end
-    assert false, "Unexpected Error Output: #{error_msg}"
+    assert_equal expected_error_msg, error_msg, message
   end
 
   ## Interaction utilities
@@ -399,10 +404,9 @@ class SassSpecRunner
   # If the runner is running in interactive mode, runs the interaction defined
   # in the block and returns the result. Otherwise, just returns the default
   # value.
-  def interact(prompt_id, default, dir = @test_case.folder, &block)
-    if @test_case.interactive?
-      relative_path = Pathname.new(dir).relative_path_from(Pathname.new(Dir.pwd))
-      print "\nIn test case: #{relative_path}"
+  def interact(prompt_id, default, &block)
+    if @options[:interactive]
+      print "\nIn test case: #{@test_case.name}"
       return SassSpec::Interactor.interact_with_memory(@@interaction_memory, prompt_id, &block)
     else
       return default
@@ -411,11 +415,39 @@ class SassSpecRunner
 
   def show_input_choice(i)
     i.choice('i', "Show me the input.") do
-      display_text_block(File.read(@test_case.input_path))
+      display_text_block(@test_case.input)
       i.restart!
     end
   end
 
+  def show_output_choice(i)
+    if @status == 0
+      i.choice('o', "Show me the output generated.") do
+        display_text_block(@output)
+        i.restart!
+      end
+
+      if @error.length > 0
+        i.choice('e', "Show me the warning generated.") do
+          display_text_block(@error)
+          i.restart!
+        end
+      end
+    else
+      i.choice('e', "Show me the error generated.") do
+        display_text_block(@error)
+        i.restart!
+      end
+    end
+  end
+
+  def migrate_warning_choice(i)
+    i.choice('W', "Migrate the error to a warning.") do
+      @test_case.rename("error", "warning")
+      throw :done
+    end
+  end
+
   def update_output_choice(i)
     i.choice('O', "Update expected output and pass test.") do
       overwrite_test!
@@ -423,17 +455,7 @@ class SassSpecRunner
     end
   end
 
-  def migrate_version_choice(i)
-    # Don't offer to migrate if the test already targets the current version.
-    return if File.basename(@test_case.folder) =~ /-#{Regexp.quote(@options[:language_version])}/
-    i.choice('V', "Migrate copy of test to pass current version.") do
-      migrate_version! || i.restart!
-      throw :done
-    end
-  end
-
   def migrate_impl_choice(i)
-    return if for_current_impl?
     i.choice('I', "Migrate copy of test to pass on #{@test_case.impl}.") do
       migrate_impl! || i.restart!
       throw :done
@@ -441,7 +463,7 @@ class SassSpecRunner
   end
 
   def todo_choice(i)
-    return if @test_case.todo? || for_current_impl?
+    return if @test_case.todo?
     i.choice('T', "Mark spec as todo for #{@test_case.impl}.") do
       change_options(add_todo: [@test_case.impl])
       throw :done
@@ -449,22 +471,18 @@ class SassSpecRunner
   end
 
   def ignore_choice(i)
-    if for_current_impl?
-      delete_choice(i)
-    else
-      i.choice('G', "Ignore test for #{@test_case.impl} FOREVER.") do
-        change_options(
-          add_ignore_for: [@test_case.impl],
-          remove_warning_todo: [@test_case.impl],
-          remove_todo: [@test_case.impl])
-        throw :done
-      end
+    i.choice('G', "Ignore test for #{@test_case.impl} FOREVER.") do
+      change_options(
+        add_ignore_for: [@test_case.impl],
+        remove_warning_todo: [@test_case.impl],
+        remove_todo: [@test_case.impl])
+      throw :done
     end
   end
 
   def delete_choice(i)
     i.choice('D', "Delete test.") do
-      if delete_dir!(@test_case.folder)
+      if delete_test!
         throw :done
       else
         i.restart!
@@ -494,107 +512,56 @@ class SassSpecRunner
     end
   end
 
-  # Deletes a directory. In interactive mode, prompts the user and returns
-  # `false` if they decide not to delete the directory.
-  def delete_dir!(dir)
-    result = interact(:delete_test, :proceed) do |i|
-      files = Dir.glob(File.join(dir, "**", "*"))
-      i.prompt("The following files will be removed:\n  * " + files.join("\n  * "))
-      i.choice('D', "Delete them.") do
-        FileUtils.rm_rf(@test_case.folder)
-      end
-      i.choice('x', "I changed my mind.") do
-      end
-    end
-    result == :proceed
-  end
-
-  # Creates a copy of the test test that's compatible with the current version.
+  # Deletes the current test case.
   #
-  # Marks the original as only being valid until the version before the current
-  # version. Marks the copy as being valid starting with the current version.
-  # Updates the copy to expect current actual results.
-  def migrate_version!
-    current_version = @options[:language_version]
-    current_version_index = SassSpec::LANGUAGE_VERSIONS.index(current_version)
-    if current_version_index == 0
-      puts "Cannot migrate test. There's no version earlier than #{current_version}."
-      return
-    end
+  # In interactive mode, prompts the user and returns `false` if they decide not
+  # to delete the test.
+  def delete_test!
+    result = interact(:delete_test, :proceed) do |i|
+      files = @test_case.dir.glob("**/*")
+      i.prompt("The following files will be removed:\n  * " + files.join("\n  * "))
 
-    start_version_index = SassSpec::LANGUAGE_VERSIONS.index(
-      @test_case.metadata.start_version.to_s)
-
-    if start_version_index >= current_version_index
-      puts "Cannot migrate test. Test does not apply to an earlier version."
-      return
-    end
-
-    previous_version = SassSpec::LANGUAGE_VERSIONS[current_version_index - 1]
-
-    new_folder = @test_case.folder + "-#{current_version}"
-
-    if File.exist?(new_folder)
-      choice = interact(:migrate_over_existing, :abort) do |i|
-        i.prompt("Target folder '#{new_folder}' already exists.")
-
-        i.choice('x', "Don't migrate the test.") do
-          return
-        end
-
-        i.choice('O', "Remove it.") do
-          unless delete_dir!(new_folder)
-            i.restart!
-          end
-        end
+      i.choice('D', "Delete them.") do
+        @test_case.dir.delete_dir!
       end
 
-      if choice == :abort
-        puts "Cannot migrate test. #{new_folder} already exists."
-        return
-      end
+      i.choice('x', "I changed my mind.") {}
     end
-
-    FileUtils.cp_r @test_case.folder, new_folder
-
-    new_test_case = SassSpec::TestCase.new(new_folder, @options)
-    change_options(end_version: previous_version)
-    change_options(new_test_case.options_path, start_version: current_version)
-
-    overwrite_test!(new_test_case)
+    result == :proceed || result == "D"
   end
 
   # Adds separate outputs for the test that are compatible with the current
   # implementation.
   def migrate_impl!
-    output, clean_output, error, status = @test_case.output
+    if @status == 0
+      if @test_case.expected != @normalized_output || @test_case.should_fail?
+        @test_case.write("output.css", @output, impl: true)
+      end
 
-    if @test_case.expected != clean_output
-      File.write(@test_case.impl_expected_path, output, binmode: true)
+      if extract_error_message(@test_case.expected_warning) != extract_error_message(@error)
+        @test_case.write("warning", @error, impl: true)
+      end
+    else
+      actual_error = @test_case.expected_error && extract_error_message(@test_case.expected_error)
+      if actual_error != extract_error_message(@error)
+        @test_case.write("error", @error, impl: true)
+      end
     end
 
-    if extract_error_message(@test_case.expected_error) != extract_error_message(error)
-      File.write(@test_case.impl_error_path, error, binmode: true)
-    end
-
-    if @test_case.expected_status != status
-      File.write(@test_case.impl_status_path, status, binmode: true)
-    end
-
-    change_options(@test_case.options_path,
-        remove_warning_todo: [@test_case.impl], remove_todo: [@test_case.impl])
+    change_options(remove_warning_todo: [@test_case.impl], remove_todo: [@test_case.impl])
   end
 
   ## Other utilities
 
-  # Returns whether the current spec targets only the implementation being
-  # tested.
-  def for_current_impl?
-    File.basename(@test_case.folder) =~ /-#{Regexp.quote(@test_case.impl)}/
+  # Returns whether the current test case is marked as TODO, but is still being
+  # run because --probe-todo was passed. These specs shouldn't produce errors
+  # when they fail.
+  def probe_todo?
+    @options[:probe_todo] && (@test_case.todo? || @test_case.warning_todo?)
   end
 
   def skip_test_case!(reason = nil)
-    msg = "Skipped #{@test_case.folder}"
+    msg = "Skipped #{@test_case.name}"
     if reason
       msg << ": #{reason}"
     else
@@ -605,6 +572,11 @@ class SassSpecRunner
 
   ANSI_ESCAPES = /[\u001b\u009b][\[()#;?]*(?:[0-9]{1,4}(?:;[0-9]{0,4})*)?[0-9A-ORZcf-nqry=><]/
   def display_text_block(text)
+    if text.empty?
+      puts "*" * 20 + " (empty) " + "*" * 20
+      return
+    end
+
     delim = "*" * text.gsub(ANSI_ESCAPES, '').lines.map{|l| l.rstrip.size}.max
 
     puts delim
@@ -612,43 +584,28 @@ class SassSpecRunner
     puts delim
   end
 
-  def overwrite_test!(test_case = @test_case)
-    output, _, error, status = test_case.output
+  def overwrite_test!
+    if @status == 0
+      @test_case.write("output.css", @output, impl: :auto)
+      @test_case.delete("error", if_exists: true, impl: :auto)
 
-    if status != 0
-      File.open(test_case.status_path, "w+", :binmode => true) do |f|
-        f.write(status)
+      if @error.empty?
+        @test_case.delete("warning", if_exists: true, impl: :auto)
+      else
+        @test_case.write("warning", @error, impl: :auto)
       end
-    elsif (File.file?(test_case.status_path))
-      File.unlink(test_case.status_path)
+    else
+      @test_case.write("error", @error, impl: :auto)
+      @test_case.delete("output.css", if_exists: true, impl: :auto)
+      @test_case.delete("warning", if_exists: true, impl: :auto)
     end
 
-    if error.length > 0
-      File.open(test_case.error_path, "w+", :binmode => true) do |f|
-        f.write(error)
-      end
-    elsif (File.file?(test_case.error_path))
-      File.unlink(test_case.error_path)
-    end
-
-    File.open(test_case.expected_path, "w+", :binmode => true) do |f|
-      f.write(output)
-    end
-
-    change_options(test_case.options_path,
-      remove_warning_todo: [test_case.impl], remove_todo: [test_case.impl])
+    change_options(remove_warning_todo: [@test_case.impl], remove_todo: [@test_case.impl])
   end
 
-  def change_options(file_or_new_options, new_options = nil)
-    if new_options
-      options_file = file_or_new_options
-    else
-      options_file = @test_case.options_path
-      new_options = file_or_new_options
-    end
-
-    existing_options = if File.exist?(options_file)
-                         YAML.load_file(options_file)
+  def change_options(new_options)
+    existing_options = if @test_case.file?("options.yml")
+                         YAML.load(@test_case.read("options.yml"))
                        else
                          {}
                        end
@@ -656,11 +613,9 @@ class SassSpecRunner
     existing_options = SassSpec::TestCaseMetadata.merge_options(existing_options, new_options)
 
     if existing_options.any?
-      File.open(options_file, "w") do |f|
-        f.write(existing_options.to_yaml)
-      end
+      @test_case.write("options.yml", existing_options.to_yaml)
     else
-      File.unlink(options_file) if File.exist?(options_file)
+      @test_case.delete("options.yml", if_exists: true)
     end
   end
 
@@ -668,7 +623,7 @@ class SassSpecRunner
   # When running sass-spec as a gem from github very long filenames can cause
   # installation issues. This checks that the paths in use will work.
   def assert_filename_length!(filename)
-    name = relative_name = filename.to_s.sub(File.expand_path(@options[:spec_directory]), "")
+    name = relative_name = filename.to_s.sub(SassSpec::SPEC_DIR, "")
     assert false, "Filename #{name} must no more than #{256 - GEMFILE_PREFIX_LENGTH} characters long" if name.size > (256 - GEMFILE_PREFIX_LENGTH)
 
     if name.size <= 100 then
@@ -708,7 +663,7 @@ class SassSpecRunner
         error_message += line
         break
       end
-      if (line =~ /DEPRECATION WARNING/)
+      if (line =~ /(DEPRECATION )?WARNING/)
         if line.rstrip.end_with?(":")
           error_message = line.rstrip + "\n"
           consume_next_line = true
@@ -727,12 +682,10 @@ class SassSpecRunner
   end
 
   def clean_debug_path(error)
-    spec_dir = File.expand_path(@options[:spec_directory])
-    url = spec_dir.gsub(/\\/, '\/')
     error.gsub(/^.*?(input.scss:\d+ DEBUG:)/, '\1')
          .gsub(/\/+/, "/")
-         .gsub(/^#{Regexp.quote(url)}\//, "/sass/sass-spec/")
-         .gsub(/^#{Regexp.quote(spec_dir)}\//, "/sass/sass-spec/")
+         .gsub(/^#{Regexp.quote(SassSpec::SPEC_DIR.gsub(/\\/, '\/'))}\//, "/sass/sass-spec/")
+         .gsub(/^#{Regexp.quote(SassSpec::SPEC_DIR)}\//, "/sass/sass-spec/")
          .gsub(/(?:\/todo_|_todo\/)/, "/")
          .gsub(/\/libsass\-[a-z]+\-tests\//, "/")
          .gsub(/\/libsass\-[a-z]+\-issues/, "/libsass-issues")
