@@ -1,5 +1,6 @@
 require 'pathname'
 require 'yaml'
+
 module SassSpec
   class TestCaseMetadata
     def self.cache
@@ -39,29 +40,28 @@ module SassSpec
 
     attr_reader :options
 
-    def initialize(test_case_dir, spec_dir = SassSpec::SPEC_DIR)
-      @test_case_dir = Pathname.new(File.expand_path(test_case_dir))
-      @spec_dir = Pathname.new(spec_dir)
-      if @test_case_dir.relative_path_from(@spec_dir).to_s.start_with?("..")
-        raise ArgumentError,
-          "Test case #{test_case_dir} is not within the spec directory #{spec_dir}"
-      end
-      @options = resolve_options(@test_case_dir).freeze
-    end
+    # The name of the test.
+    #
+    # This is a standardized format of the test's directory name.
+    attr_reader :name
 
-    def name
-      @test_case_dir.relative_path_from(Pathname.new(Dir.pwd)).to_s
+    # Parses metadata for the test case at the given SassSpec::Directory.
+    def initialize(test_case_dir)
+      @name = test_case_dir.to_s
+      @options = resolve_options(test_case_dir).freeze
     end
 
     def _resolve_options(dir)
-      return {} if dir.relative_path_from(@spec_dir).to_s == "."
-      parent_options = resolve_options(dir.parent)
-      options_file = dir + "options.yml"
-      self_options = if options_file.exist?
-                       YAML.load_file(options_file.to_s)
+      return {} unless parent = dir.parent
+
+      parent_options = resolve_options(parent)
+      self_options = if dir.file?("options.yml")
+                       YAML.load(dir.read("options.yml"))
                      else
                        {}
                      end
+      raise "#{dir.path}/options.yml is not a map!" unless self_options.is_a?(Hash)
+
       rv = parent_options.merge(self_options) do |key, parent_value, self_value|
         if ACCUMULATED_OPTIONS.include?(key)
           (Array(parent_value) + Array(self_value)).uniq
@@ -73,7 +73,7 @@ module SassSpec
     end
 
     def resolve_options(dir)
-      self.class.cache[dir] ||= _resolve_options(dir).freeze
+      self.class.cache[dir.path] ||= _resolve_options(dir).freeze
     end
 
     def todo?(impl)
@@ -102,48 +102,12 @@ module SassSpec
       @options[:ignore_warning_for] || []
     end
 
-    def output_style
-      @options[:output_style]
-    end
-
     def precision
       @options[:precision]
     end
 
-    def clean_output?
-      !!@options[:clean]
-    end
-
-    def start_version
-      unless @options[:start_version]
-        raise ArgumentError.new("#{name} does not specify a start version.")
-      end
-      @start_version ||= Gem::Version.new(@options[:start_version])
-    end
-
-    def end_version
-      unless defined?(@end_version)
-        @end_version = if @options[:end_version]
-                         Gem::Version.new(@options[:end_version])
-                       else
-                         nil
-                       end
-      end
-      @end_version
-    end
-
-    def valid_for_version?(version)
-      version = Gem::Version.new(version) if version.is_a?(String)
-      valid = start_version <= version 
-      if end_version
-        valid &&= version <= end_version
-      end
-      valid
-    end
-
     def valid_for_impl?(impl)
       !ignore_for?(impl)
     end
-
   end
 end
